{ open Parser 
  let dbl_char c = (String.make 1 c) ^ (String.make 1 c) (* Turns 'x' into "xx" *)
}

let digit = ['0'-'9']
let exp = ['e' 'E'] ['+' '-']? digit+
let float = ((digit+ '.') | digit* '.' digit+) exp? | digit+ exp
let hex = digit | ['a'-'f' 'A'-'F']
let nl = [' ' '\r' '\t']* '\n' | [' ' '\r' '\t']* ';' [^ '\n']* '\n' (* Single-line comments are also newlines *)

rule token = parse
    [' ' '\r' '\t'] { token lexbuf }
  | ('"' ([^ '"' ]* as lit) '"')
  | ('\'' ([^ '\'' ]* as lit) '\'') { STRING(lit) }
  | digit+ as lit { INT(int_of_string lit) }
  | float as lit { FLOAT(float_of_string lit) }
  | '#' (hex hex as r) (hex hex as g) (hex hex as b) { COLOR(r, g, b, 1.0) }
  | '#' (hex hex as r) (hex hex as g) (hex hex as b) '@' (float as a) { COLOR(r, g, b, float_of_string a) }
  | '#' (hex as r) (hex as g) (hex as b) { COLOR(dbl_char r, dbl_char g, dbl_char b, 1.0) }
  | '#' (hex as r) (hex as g) (hex as b) '@' (float as a) { COLOR(dbl_char r, dbl_char g, dbl_char b, float_of_string a) }
  | "+" { PLUS }
  | "-" { MINUS }
  | "*" { TIMES }
  | "/" { DIV }
  | "//" { DIVDIV }
  | "^" { EXP }
  | "%" { PERCENT }
  | "+=" { PLUSEQ }
  | "-=" { MINUSEQ }
  | "*=" { TIMESEQ }
  | "/=" { DIVEQ }
  | "//=" { DIVDIVEQ }
  | "^=" { EXPEQ }
  | "%=" { PERCENTEQ }
  | "=" { EQ }
  | "==" { EQEQ }
  | "!=" { NEQ }
  | ">" { GT }
  | "<" { LT }
  | ">=" { GTE }
  | "<=" { LTE }
  | "!" { NOT }
  | "&" { AND }
  | "|" { OR }
  | "?" { QUESTION }
  | "." { DOT }
  | ":" { COLON }
  | "," { COMMA }
  | "(" nl* { LPAREN }
  | nl* ")" { RPAREN }
  | "{" nl* { LBRACE }
  | nl* "}" { RBRACE }
  | "[" nl* { LBRACKET }
  | nl* "]" { RBRACKET }
  | "<<" nl* { LANGLE }
  | nl* ">>" { RANGLE }
  | nl+ { NL }
  | "param" { PARAM }
  | "comp" { COMP }
  | "var" { VAR }
  | "if" { IF }
  | nl* [' ' '\r' '\t']*  "elseif" { ELSEIF }
  | nl* [' ' '\r' '\t']*  "else" { ELSE }
  | "do" { DO }
  | "while" { WHILE }
  | "true" { TRUE }
  | "false" { FALSE }
  | "return" { RETURN }
  | "figure" { FIGURE }
  | "int" { T_INT }
  | "float" { T_FLOAT }
  | "string" { T_STRING }
  | "bool" { BOOL }
  | "func" { FUNC }
  | "color" { T_COLOR }
  | "pair" { PAIR }
  | ['a'-'z' 'A'-'Z' '_'] ['a'-'z' 'A'-'Z' '0'-'9' '_']* as lit { ID(lit) }
  | eof { EOF }
  | _ as char { raise (Failure("Illegal character " ^
				 Char.escaped char)) }