(* strings.ml
  Contains functions for pretty printing
*)

open Ast

let of_list = function
  [] -> ""
| hd::tl -> List.fold_left (fun s x -> s ^ ", " ^ x) hd tl

let rec of_type = function
  IntT -> "int"
| FloatT -> "float"
| StringT -> "string"
| BoolT -> "bool"
| ColorT -> "color"
| PairT -> "pair"
| FigureT(Some(id)) -> id ^ "{}"
| FigureT(None) -> "figure{}"
| VoidT -> ""
| FigDefT -> "figureDefinition"
| ListT(typ) -> (of_type typ) ^ "[]"
| FuncT(ret, args) -> (of_type ret) ^ "(" ^
    of_list (List.map (fun t -> of_type t) args) ^ ")"

let of_arop = function
  Plus -> "+"
| Minus -> "-"
| Times -> "*"
| Div -> "/"
| Divdiv -> "//"
| Exp -> "^"
| Percent -> "%"

let of_logop = function
  Eqeq -> "=="
| Neq -> "!="
| Gt -> ">"
| Lt -> "<"
| Gte -> ">="
| Lte -> "<="
| And -> "&"
| Or -> "|"

let of_unop = function
  Neg -> "-"
| Not -> "!"
| Question -> "?"

let of_scope scope = (match scope.outer with
  None -> "none" | Some(int) -> string_of_int int) ^
  (match scope.capture with None -> "" | Some(captures) -> "(Captures " ^
    StringMap.fold (fun sym _ s -> s ^ sym ^ ", ") captures "" ^
  ")") ^
  " -> " ^ (StringMap.fold
  (fun sym ty s -> s ^ sym ^ ": " ^ (of_type ty) ^ ", ") scope.symbols "")

let of_scopes scopes = 
    let counter = ref (-1) in
    List.fold_left (fun s scope -> counter := !counter + 1;
	s ^ (string_of_int !counter) ^ ") " ^
	(match scope.figure with
  None -> "" | Some({name=id; _}) -> id ^ ", ") ^ of_scope scope ^
  "\n") "" scopes
